import { Component, Input, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { ClientViewComponent } from "../client-view/client-view.component";
import { CdkDragEnd, CdkDragMove } from '@angular/cdk/drag-drop';

interface GroupedClients {
  organizationalUnitName: string;
  clientRows: any[][];
}

@Component({
  selector: 'app-classroom-view',
  templateUrl: './classroom-view.component.html',
  styleUrls: ['./classroom-view.component.css']
})
export class ClassroomViewComponent implements OnInit {
  @Input() clients: any[] = [];
  @Input() pcInTable: number = 5;
  groupedClients: GroupedClients[] = [];

  constructor(public dialog: MatDialog) {}

  ngOnInit(): void {
    this.groupClientsByOrganizationalUnit();
    this.initializeClientPositions();
  }

  ngOnChanges(): void {
    this.groupClientsByOrganizationalUnit();
    this.initializeClientPositions();
  }

  groupClientsByOrganizationalUnit(): void {
    const grouped = this.clients.reduce((acc, client) => {
      const ouName = client.organizationalUnit.name;
      if (!acc[ouName]) {
        acc[ouName] = [];
      }
      acc[ouName].push(client);
      return acc;
    }, {});

        /* this.groupedClients = Object.keys(grouped).map(ouName => ({
      organizationalUnitName: ouName,
      clientRows: this.chunkArray(grouped[ouName], this.pcInTable)
    })); */

    this.groupedClients = [{
      "organizationalUnitName": "Aula01",
      "clientRows": [
        [
          {
            "@id": "/clients/0190b603-3e40-7256-90dd-ea04b4ea0e41",
            "@type": "Client",
            "name": "PC1",
            "position": `{"x": -165, "y": -93}`,
            "type": "client",
            "ip": "127.0.0.1",
            "mac": "00:00:00:00:1",
            "serialNumber": "1",
            "netiface": "eth0",
            "netDriver": "generic",
            "organizationalUnit": {
              "@id": "/organizational-units/0190b602-35f2-7026-a42a-cc310ed07921",
              "@type": "OrganizationalUnit",
              "name": "Aula01",
              "type": "classroom",
              "networkSettings": {
                "@id": "/network-settings/0190b602-35f2-7026-a42a-cc310f5c8648",
                "@type": "NetworkSettings",
                "validation": false,
                "uuid": "0190b602-35f2-7026-a42a-cc310f5c8648",
                "id": 2
              },
              "uuid": "0190b602-35f2-7026-a42a-cc310ed07921",
              "id": 2
            },
            "partitions": [],
            "createdAt": "2024-07-15T10:49:44+00:00",
            "createdBy": "ogadmin",
            "uuid": "0190b603-3e40-7256-90dd-ea04b4ea0e41",
            "id": 1
          },
          {
            "@id": "/clients/0190b603-e73e-7033-bba1-309d4672f581",
            "@type": "Client",
            "name": "PC2",
            "position": `{"x": -65, "y": 55}`,
            "type": "client",
            "ip": "123.0.0.2",
            "mac": "00:00:00:00:2",
            "serialNumber": "2",
            "netiface": "eth0",
            "organizationalUnit": {
              "@id": "/organizational-units/0190b602-35f2-7026-a42a-cc310ed07921",
              "@type": "OrganizationalUnit",
              "name": "Aula01",
              "type": "classroom",
              "networkSettings": {
                "@id": "/network-settings/0190b602-35f2-7026-a42a-cc310f5c8648",
                "@type": "NetworkSettings",
                "validation": false,
                "uuid": "0190b602-35f2-7026-a42a-cc310f5c8648",
                "id": 2
              },
              "uuid": "0190b602-35f2-7026-a42a-cc310ed07921",
              "id": 2
            },
            "partitions": [],
            "createdAt": "2024-07-15T10:50:27+00:00",
            "createdBy": "ogadmin",
            "uuid": "0190b603-e73e-7033-bba1-309d4672f581",
            "id": 2
          },
        ]
      ]
    }]
  }

  initializeClientPositions(): void {
    this.groupedClients.forEach(group => {
      group.clientRows.forEach(row => {
        row.forEach(client => {
          const position = JSON.parse(client.position);
          client.dragPosition = { x: position.x, y: position.y };
        });
      });
    });
  }

  chunkArray(arr: any[], chunkSize: number): any[][] {
    const chunks = [];
    for (let i = 0; i < arr.length; i += chunkSize) {
      chunks.push(arr.slice(i, i + chunkSize));
    }
    return chunks;
  }

  handleClientClick(client: any): void {
    const dialogRef = this.dialog.open(ClientViewComponent, { data: { client }, width: '800px', height:'700px' });
  }

  onDragMoved(event: CdkDragMove<any>): void {
    console.log('Elemento movido:', event.source.element.nativeElement);
    console.log('Posición actual:', event.pointerPosition);
    console.log('Posición relativa:', event.distance);
  }
}
